@@ -64,7 +64,8 @@ class Tomography(Experiment):
     def show_pattern(self):
         if self.AcousticFields is None:
             raise ValueError("AcousticFields is not initialized. Please generate the system matrix first.")
-        # Collect entries as a list of tuples
+
+        # Collect and sort entries
         entries = []
         for field in self.AcousticFields:
             if field.waveType != WaveType.StructuredWave:
@@ -72,55 +73,80 @@ class Tomography(Experiment):
             pattern = field.pattern
             entries.append((
                 (pattern.space_0, pattern.space_1, pattern.move_head_0_2tail, pattern.move_tail_1_2head),
-                pattern.activeList,  # hex_str
+                pattern.activeList,
                 field.angle
             ))
-        # Sort entries
-        entries.sort(
-            key=lambda x: (
-                -(x[0][0] + x[0][1]),  # Total length descending
-                -max(x[0][0], x[0][1]), # Max(space_0, space_1) descending
-                -x[0][0],              # space_0 descending
-                -x[0][2],              # move_head_0_2tail descending
-                x[0][3]                # move_tail_1_2head ascending
-            )
-        )
+
+        entries.sort(key=lambda x: (
+            -(x[0][0] + x[0][1]),
+            -max(x[0][0], x[0][1]),
+            -x[0][0],
+            -x[0][2],
+            x[0][3]
+        ))
+
         # Extract data
         hex_list = [hex_str for _, hex_str, _ in entries]
         angle_list = [angle for _, _, angle in entries]
-        space_data = [t for t, _, _ in entries]
-        # Convert hex strings to binary columns
+
         def hex_string_to_binary_column(hex_str):
             bits = ''.join(f'{int(c, 16):04b}' for c in hex_str)
             return np.array([int(b) for b in bits], dtype=np.uint8).reshape(-1, 1)
+
         bit_columns = [hex_string_to_binary_column(h) for h in hex_list]
         image = np.hstack(bit_columns)
-        height = image.shape[0]
-        # Plot
-        _, ax = plt.subplots(figsize=(12, 10))
-        ax.imshow(image, cmap='gray', aspect='auto')
-        ax.set_title("Scan configuration", fontsize='large')
-        ax.set_xlabel("Wave", fontsize='medium')
-        ax.set_ylabel("Transducer activation", fontsize='medium')
-        # Plot angle markers
-        angle_min = -20.2
-        angle_max = 20.2
+        print(image)  # Doit être un tableau de 1 partout
+
+        height, width = image.shape
+
+        # Create figure with compact size
+        fig, ax = plt.subplots(figsize=(5, 4))
+        plt.subplots_adjust(left=0.1, right=0.95, top=0.9, bottom=0.2)
+
+        # Plot binary pattern
+        im = ax.imshow(image, cmap='binary', aspect='auto', interpolation='none', vmin=0, vmax=1)
+
+        ax.set_title("Scan Configuration", fontsize=12, pad=10, weight='bold')
+        ax.set_xlabel("Wave Index", fontsize=10, labelpad=8)
+        ax.set_ylabel("Transducer Activation", fontsize=10, labelpad=8)
+        yticks_positions = np.arange(0, height)  # Positions des ticks (0 à 191)
+        yticks_labels = np.arange(1, height + 1)  # Labels de 1 à 192
+
+        ax.set_yticks(yticks_positions)
+        ax.set_yticklabels(yticks_labels, fontsize=8)
+        # Plot angle markers (bigger and bolder)
+        angle_min, angle_max = -20.2, 20.2
         center = height / 2
         scale = height / (angle_max - angle_min)
         for i, angle in enumerate(angle_list):
             y = round(center - angle * scale)
             if 0 <= y < height:
-                ax.plot(i, y - 0.5, 'r.', markersize=5)
+                ax.plot(i, y - 0.5, 'ro', markersize=4, alpha=0.7)  # Points rouges plus gros
+
         ax.set_ylim(height - 0.5, -0.5)
-        # Twin axis for angle labels
+
+        # Twin axis for angles (with larger font)
         ax2 = ax.twinx()
         ax2.set_ylim(ax.get_ylim())
-        yticks_angle = np.linspace(20, -20, 9)
+        yticks_angle = np.linspace(20, -20, 5)  # 5 ticks pour plus de détails
         yticks_pos = np.interp(yticks_angle, [angle_min, angle_max], [height - 0.5, -0.5])
         ax2.set_yticks(yticks_pos)
-        ax2.set_yticklabels([f"{a:.1f}°" for a in yticks_angle])
-        ax2.set_ylabel("Angle [degree]", fontsize='medium', color='r')
-        ax2.tick_params(axis='y', colors='r')
+        ax2.set_yticklabels([f"{a:.1f}°" for a in yticks_angle], fontsize=9, color='r')
+        ax2.set_ylabel("Angle [°]", fontsize=11, color='r', labelpad=10)
+        ax2.tick_params(axis='y', colors='r', labelsize=9, width=1.5, length=5)
+
+        # Make axes thicker
+        ax.spines['left'].set_linewidth(1.5)
+        ax.spines['bottom'].set_linewidth(1.5)
+        ax2.spines['right'].set_linewidth(1.5)
+
+        # Add grid (thicker lines)
+        ax.grid(True, linestyle='--', alpha=0.4, color='gray', linewidth=0.5)
+        ax.set_xticks(np.linspace(0, width-1, 6))  # Plus de ticks sur l'axe x
+        ax.set_yticks(np.linspace(0, height-1, 6))  # Plus de ticks sur l'axe y
+        ax.tick_params(axis='both', which='both', labelsize=8, width=1.5, length=4)
+
+        plt.tight_layout()
         plt.show()
 
     def plot_angle_frequency_distribution(self):
