@@ -117,7 +117,6 @@ class Experiment(ABC):
         else:
             self.AOsignal_withoutTumor = noiseSignals
 
-
     def reduceDims(self, mode='avg'):
         """
         Réduit les dimensions T, X, Z d'un numpy array (T, X, Z) par 2 en utilisant une convolution.
@@ -296,43 +295,39 @@ class Experiment(ABC):
 
         return AOsignal_matrix
 
-    def _saveAOsignals_Castor(self, save_directory, withTumor=True):
+    def saveAOsignals_Castor(self, save_directory, withTumor=True):
         if withTumor:
+            AO_signal = self.AOsignal_withTumor
             cdf_location = os.path.join(save_directory, "AOSignals_withTumor.cdf")
             cdh_location = os.path.join(save_directory, "AOSignals_withTumor.cdh")
         else:
+            AO_signal = self.AOsignal_withoutTumor
             cdf_location = os.path.join(save_directory, "AOSignals_withoutTumor.cdf")
             cdh_location = os.path.join(save_directory, "AOSignals_withoutTumor.cdh")
 
         info_location = os.path.join(save_directory, "info.txt")
-        nScan = self.AOsignal.shape[1]
+        nScan = AO_signal.shape[1]
 
         with open(cdf_location, "wb") as fileID:
-            for j in range(self.AOsignal.shape[1]):
-                active_list = self.AcousticFields[j].pattern.activeList
-                angle = self.AcousticFields[j].angle
-                active_list_str = ''.join(map(str, active_list))
-                nb_padded_zeros = (4 - len(active_list_str) % 4) % 4
-                active_list_str += '0' * nb_padded_zeros
-                active_list_hex = ''.join([hex(int(active_list_str[i:i+4], 2))[2:] for i in range(0, len(active_list_str), 4)])
-
+            for j in range(AO_signal.shape[1]):
+                active_list_hex = self.AcousticFields[j].pattern.activeList
                 for i in range(0, len(active_list_hex), 2):
-                    byte_value = int(active_list_hex[i:i + 2], 16)
+                    byte_value = int(active_list_hex[i:i+2], 16)
                     fileID.write(byte_value.to_bytes(1, byteorder='big'))
-
+                angle = self.AcousticFields[j].angle
                 fileID.write(np.int8(angle).tobytes())
-                fileID.write(self.AOsignal[:, j].astype(np.float32).tobytes())
+                fileID.write(AO_signal[:, j].astype(np.float32).tobytes())
 
         header_content = (
             f"Data filename: AOSignals.cdf\n"
             f"Number of events: {nScan}\n"
-            f"Number of acquisitions per event: {self.AOsignal.shape[1]}\n"
+            f"Number of acquisitions per event: {AO_signal.shape[1]}\n"
             f"Start time (s): 0\n"
             f"Duration (s): 1\n"
             f"Acquisition frequency (Hz): {1/self.AcousticFields[0].kgrid.dt}\n"
             f"Data mode: histogram\n"
             f"Data type: AOT\n"
-            f"Number of US transducers: {self.params['num_elements']}"
+            f"Number of US transducers: {self.params.acoustic['num_elements']}"
         )
 
         with open(cdh_location, "w") as fileID:
