@@ -610,7 +610,7 @@ class AlgebraicRecon(Recon):
         print(f"Reconstruction results loaded from {os.path.join(self.saveDir, 'results')}")
 
     def normalizeSMatrix(self):
-        self.SMatrix = self.SMatrix / (self.params.acoustic['voltage']*self.params.acoustic['sensitivity'])
+        self.SMatrix = self.SMatrix / (self.experiment.params.acoustic['voltage']*self.experiment.params.acoustic['sensitivity'])
 
     # PRIVATE METHODS
 
